/* getargs.c - command line argument processor for C programs */
/*		J.E. Boyd - inspired by  A. I. Holub (c) 1985 */

#include <stdio.h>
#include <stdlib.h>
#include "getargs.h"


/* set an argument - argp points at the argument table
	entry corresponding to *linep.  return liep, updated to point past
	the argument being set */

static char *setarg( argp, linep )
ARG *argp;
char *linep;
{

	switch( argp->type ) {
	case INTEGER:
		*argp->variable = atoi( linep );
		break;

	case FLOAT:
		*(float *)argp->variable = atof( linep );
		break;

	case BOOLEAN:
		if ( strcmp( linep, "off" ) == 0 || strcmp( linep, "OFF" ) == 0 )
			*argp->variable = 0;
		else
		if ( strcmp( linep, "on" ) == 0 || strcmp( linep, "ON" ) == 0 )
			*argp->variable = 1;
		else
			fprintf( stderr, "getargs: boolean argument must be on/off\n" );
		break;

	case CHARACTER:
		*(char *)argp->variable = *linep;

	case STRING:
		*(char **)argp->variable = linep;
		break;

	default:
		fprintf( stderr, "getargs:  bad argument type\n" );
		break;
	}
}

/* return pointer to argument table entry correspoonding
	to c (or 0 if c isn't in table) */

static ARG *findarg( c, tabp, tabsize )
char *c;
int tabsize;
ARG *tabp;
{
	for( ; --tabsize >= 0; tabp++ )
		if ( strcmp( tabp->arg, c ) == 0 )
			return tabp;

	return 0;
}

/* print the argtab in the form:
	-<arg> <errmsg>		(value is <*variable>) */

pr_usage( tabp, tabsize )
ARG *tabp;
int tabsize;
{
	for( ; --tabsize >= 0; tabp++ ) {
		switch( tabp->type ) {
		case INTEGER:
			fprintf( stderr, "-%5s <integer> %-40s ( value is ",
				tabp->arg, tabp->errmsg );
			fprintf( stderr, "%d )\n", *(tabp->variable) );
			break;

		case FLOAT:
			fprintf( stderr, "-%5s <float>   %-40s ( value is ",
				tabp->arg, tabp->errmsg );
			fprintf( stderr, "%g )\n", *((float *)tabp->variable) );
			break;

		case BOOLEAN:
			fprintf( stderr, "-%5s <on/off>  %-40s ( value is ",
				tabp->arg, tabp->errmsg );
			fprintf( stderr, "%s )\n", *(tabp->variable)
					? "on" : "off" );
			break;

		case CHARACTER:
			fprintf( stderr, "-%5s <char>    %-40s ( value is ",
				tabp->arg, tabp->errmsg );
			fprintf( stderr, "%c )\n", *(tabp->variable) );
			break;

		case STRING:
 			fprintf( stderr, "-%5s <string>  %-40s ( value is ",
				tabp->arg, tabp->errmsg );
			fprintf( stderr, "<%s> )\n", *((char **)tabp->variable) );
			break;

		}
	}
}



#define		ERRMSG		"Illegal argument<%s>.  Legal arguments are:\n\n"

/* process command line arguments stripping all command line
	switches out of argv.  return a new argc.  If an error is found
	exit( 1 ) is called (getargs won't return) and a suage message
	is printed showing all arguments in the table */

int getargs( argc, argv, tabp, tabsize )
int argc, tabsize;
char **argv;
ARG *tabp;
{
register int nargc;
register char **nargv, *p;
register ARG *argp;

	nargc = 1;
	for( nargv = ++argv; --argc > 0; argv++ ) {
		if ( **argv != '-' ) {
			*nargv++ = *argv;
			nargc++;
		}
		else {
			p = *argv;

			if ( *(p+1) == '-' ) {
				pr_usage( tabp, tabsize );
				exit( 1 );
			}
			else {
				if ( argp = findarg( p+1, tabp, tabsize ) ) {
					argv++;
					argc--;
					if ( argc <= 0 ) {
						pr_usage( tabp, tabsize );
						exit( 0 );
					}
					p = *argv;
					p = setarg( argp, p );
				}
				else {
					fprintf( stderr, ERRMSG, p+1 );
					pr_usage( tabp, tabsize );
					exit( 0 );
				}
			}
		}
	}
	return nargc;
}
